@@ -1,11 +1,8 @@
 import orjson
 import os
-import glob
 import io
 import urllib
 
-from collections import OrderedDict
-
 from label_studio.utils.io import iter_files
 
 
@@ -27,7 +24,6 @@ class Tasks(object):
         base_url = HOSTNAME + '/'
         image_url_path = base_url + urllib.parse.quote('data/' + filename)
         image_local_url = '{image_url_path}?{params}'.format(image_url_path=image_url_path, params=params)
-        print(image_local_url)
         return {
             'id': task_id,
             'task_path': filepath,
@@ -35,7 +31,7 @@ class Tasks(object):
         }
 
     def from_dict(self, d, task_id=0):
-        task = OrderedDict()
+        task = {}
         data = d['data'] if 'data' in d else d
         task[task_id] = {'id': task_id, 'data': data}
         if 'predictions' in data:
@@ -45,13 +41,19 @@ class Tasks(object):
             task[task_id]['predictions'] = d['predictions']
         return task
 
+    def from_list_of_dicts(self, l, start_task_id=0):
+        tasks = {}
+        for i, t in enumerate(l):
+            tasks.update(self.from_dict(t, start_task_id + i))
+        return tasks
+
     def from_json_file(self, path, start_task_id=0):
         with open(path) as f:
             json_body = orjson.loads(f.read())
 
             # multiple tasks in file
             if isinstance(json_body, list):
-                tasks = OrderedDict()
+                tasks = {}
                 task_id = start_task_id
                 for d in json_body:
                     tasks.update(self.from_dict(d, task_id))
@@ -68,13 +70,13 @@ class Tasks(object):
                 raise Exception('Unsupported task data:', path)
 
     def from_dir_with_json_files(self, path):
-        tasks = OrderedDict()
+        tasks = {}
         for f in iter_files(path, ext='.json'):
             tasks.update(self.from_json_file(f, start_task_id=len(tasks)))
         return tasks
 
     def from_text_file(self, path, data_key, start_task_id=0):
-        tasks = OrderedDict()
+        tasks = {}
         task_id = start_task_id
         with io.open(path) as f:
             for line in f:
@@ -82,15 +84,17 @@ class Tasks(object):
         return tasks
 
     def from_dir_with_text_files(self, path, data_key):
-        tasks = OrderedDict()
+        tasks = {}
         for f in iter_files(path, ext=''):
             tasks.update(self.from_text_file(f, data_key, start_task_id=len(tasks)))
+        return tasks
 
     def _from_dir_with_local_resources(self, path, data_key, data_type):
-        tasks = OrderedDict()
+        tasks = {}
         for f in iter_files(path, ext=self._allowed_extensions[data_type]):
             task_id = len(tasks) + 1
             tasks[task_id] = self._create_task_with_local_uri(f, data_key, task_id)
+        return tasks
 
     def from_dir_with_image_files(self, path, data_key):
         return self._from_dir_with_local_resources(path, data_key, 'Image')
